/* _______________________________________________________________________
	Trinity.h - Core Trinity API from Eskape Labs
	
	first created: 6/22/2000
	
	copyright  1999-2001 by David DeHaven - all rights reserved.
 _______________________________________________________________________*/

#ifndef _TrinityAPI_
#define _TrinityAPI_

	#include <Trinity/TrinityCommon.h>

#ifdef __cplusplus
extern "C" {
#endif

/*______________________________________________________________________
	Setting up and terminating connections
	
	TRInitialize MUST be called prior to making any other call
	TRTerminate must be called to clean up when you are finished
  ______________________________________________________________________*/

TrinityErr TRInitialize();
void TRTerminate();

/*______________________________________________________________________
	Buffers allocated inside the Trinity framework must be deallocated
	using this call.
  ______________________________________________________________________*/

void TRFree(void *ptr);

/*______________________________________________________________________
	Trinity Driver communications callback
		This call installs the default notification callback procs.
	
	When opening a connection to a pipe, setting the notification proc
	to NULL will default to the proc supplied by this routine, then none
	if this call has not been made or no proc was specified.
  ______________________________________________________________________*/

TrinityErr TRInstallNotifyProcs(TRTrinityNotifyProc mainNotify, TRFinishProc finishProc, TRPipeNotifyProc notifyProc);

/*______________________________________________________________________
	Accessing the device list and getting device interface information
  ______________________________________________________________________*/

long TRGetDeviceCount();
TrinityErr TRGetDeviceList(TRDeviceRef **deviceRefList, unsigned long *deviceCount);

/*______________________________________________________________________
	The client owns the memory used by the tree, you can do whatever you
	need with it, then dispose of using TRFree. The tree is contained in a
	single block of memory, so simply calling free(deviceTreePtr) will work.
  ______________________________________________________________________*/

TrinityErr TRGetDeviceInfo(TRDeviceRef deviceRef, TRDeviceInfoPtr *info, unsigned long *size);

/*______________________________________________________________________
	Feature and configuration control routines
  ______________________________________________________________________*/

TrinityErr TRGetActiveConfiguration(TRDeviceRef deviceRef, unsigned long featureID, unsigned long *activeConfig);
TrinityErr TRActivateConfiguration(TRDeviceRef deviceRef, unsigned long configID);
TrinityErr TRDeactivateConfiguration(TRDeviceRef device, unsigned long configID);

/*______________________________________________________________________
	Accessing pipes in the device
  ______________________________________________________________________*/

	/*
		The finish proc and notify proc provided here will override the 
		default finish and pipe notify callbacks installed with
		TRInstallNotifyProcs.
		
		userData is passed to the callbacks when they are fired, each
		pipe reference you create has it's own userData.
	 */
TrinityErr TROpenPipe(TRPipeRef *outPipeRef, TRDeviceRef deviceRef, unsigned long pipeID);
TrinityErr TRClosePipe(TRPipeRef pipeRef);

	/*
		Callback accessors - use these to get or set the specified callback and userData argument
		
		Each callback has it's own userData value that is passed back to that callback
		If the default callback is used, the userData value is set to NULL
	 */
TrinityErr TRGetPipeFinishProc(TRPipeRef pipeRef, TRFinishProc *outProc, void **userData);
TrinityErr TRSetPipeFinishProc(TRPipeRef pipeRef, TRFinishProc finishProc, void *userData);

TrinityErr TRGetPipeNotifyProc(TRPipeRef pipeRef, TRPipeNotifyProc *outProc, void **userData);
TrinityErr TRSetPipeNotifyProc(TRPipeRef pipeRef, TRPipeNotifyProc notifyProc, void *userData);

	/* resets pipe to it's default state, use sparingly, if at all. Some drivers may ignore this completely */
TrinityErr TRResetPipe(TRPipeRef pipeRef);


/*
	In either TRSetPipeSetting or TRGetPipeSetting, the data argument has two uses:
	
	 - if dataSize is zero, then value stored in data is considered the actual value
	 - if dataSize is non-zero, then the value in data is a pointer to a buffer containing the data
	
	When you get a buffer using TRGetPipeSetting, you are responsible for deallocating the buffer
	with TRFree when you are finished with it. Buffers passed to TRSetPipeSetting can either be
	allocated or pointers to local variables (stack based typically), the framework does not attempt
	to free the data, that is up to you.
*/
TrinityErr TRSetPipeSetting(TRPipeRef pipeRef, unsigned long selector, unsigned long data, unsigned long dataSize);
TrinityErr TRGetPipeSetting(TRPipeRef pipeRef, unsigned long selector, unsigned long *data, unsigned long *dataSize);

/*______________________________________________________________________
	Pipe I/O routines
  ______________________________________________________________________*/

	/*
		This call tells the driver to prepare to begin data I/O, each client
		that expects data calls this, then calls TRReleasePipe when finished
		
		Note that other clients may change settings while data is being
		captured and may be sending or receiving data at the same time.
		
		Some pipes will only allow a single connection, such as most video
		output pipes. In this case, if a connection is already established,
		then calling TRPreparePipe will fail with kTRPipeBusyErr. This is
		completely dependent on the driver. 
		
		Some external devices may be powered on or brought out of sleep mode
		when this call is made, so this call may take a bit of time to
		complete.
	 */
TrinityErr TRPreparePipe(TRPipeRef pipeRef);

	/*
		Frees up connection info and tells the hardware driver that you are
		done working with data from this pipe. Typically (for external
		devices), when the last client has released, the hardware is shut down
		or put in low power mode to conserve energy.
	 */
TrinityErr TRReleasePipe(TRPipeRef pipeRef);

/* Continuous capture routines */
	/*
		Begin capturing data continuously
		You must specify a valid buffer list or this will fail
		When the finish proc is called, the buffer that was affected will be passed
	 */
TrinityErr TRStartPipe(TRPipeRef pipeRef, TRBufferListPtr bufferList);
TrinityErr TRStopPipe(TRPipeRef pipeRef);	/* stop... */
TrinityErr TRPausePipe(TRPipeRef pipeRef);	/* Temporarily stop I/O for this client only on the given pipe */
TrinityErr TRResumePipe(TRPipeRef pipeRef);

/* asynchronous I/O routines */
	/*
		You must supply a buffer pointer for these calls, you can stack as many
		calls as memory will allow but you should not queue any buffer more
		than once.
		
		If you must access or change a buffer once you've queued it for a read
		or write, you must call TRFlushPipe before accessing that buffer and
		then re-enqueue your async reads or writes.
		
		You can call TRReadPipe or TRWritePipe inside the callback to enqueue
		the buffer once you are finished with it, but be aware that spending a
		long amount of time inside the callback will block other callbacks from
		firing.
	 */
TrinityErr TRReadPipe(TRPipeRef pipeRef, TRBufferPtr dataBuffer);
TrinityErr TRWritePipe(TRPipeRef pipeRef, TRBufferPtr dataBuffer);

	/* this call flushes all queued async read/writes and flushes any data being accumulated by the driver */
TrinityErr TRFlushPipe(TRPipeRef pipeRef);

/*______________________________________________________________________
	Buffer management tools
  ______________________________________________________________________*/

TrinityErr TRCreateBufferList(TRBufferListPtr *outList, long bufferCount, unsigned long bufferSize); /* all buffers will be the same size */
void TRDisposeBufferList(TRBufferListPtr inList);

TrinityErr TRCreateBuffer(TRBufferPtr *outBuffer, unsigned long bufferSize);
void TRDisposeBuffer(TRBufferPtr inBuffer);

/*
	TRBufferRetain/TRBufferRelease - Simple retain/release mechanism
	
	Each call to TRBufferRetain will increment a lock count. If the lock
	count is non-zero, the driver will not attempt to use the buffer.
	
	Each call to TRBufferRetain must be balanced with a call to TRBufferRelease.
	
	When a pipe finished callback is fired, the buffer given is retained once
	to prevent the driver from using it until the client is done with the buffer
	data. When you're finished with the buffer, call TRBufferRelease to allow
	the driver to access that buffer.
 */
TrinityErr TRBufferRetain(TRBufferPtr buffer);
TrinityErr TRBufferRelease(TRBufferPtr buffer);

#ifdef __cplusplus
}
#endif

#endif /* _TrinityAPI_ */

/*______________________________________________________________________*/
